# human_template = """
# Write and call a function which shows text by streamlit text element code depending on the instruction below:
# Suppose that, streamlit has been imported by "import streamlit as st" so you don't need to import it.
#
# The function arguments are "{args}".
# You can assume that the variables with the same names are already defined.
# So you don't need to give dummy variables to the function while calling.
# You should use "{args}" to call the function
# In the function, also add a descriptive part next to the "{args}"
# After defining the function, call it with "{args}".
# Assume that, they have been already defined but only call the function by checking if the is not None and len(str(the)) > 0 input is not an empty string.
# use st.markdown to show the main text
#
# Here is the part of the code that you are supposed to continue:
# {code_snippets}
#
# Instruction:{instruction}
# Streamlit Code:
# """

system_template = """
You cannot use other st.func_name even if it is image or table or any kind. You are supposed to select one of most appropriate one in the followings:
[st.markdown, st.header, st.subheader, st.caption, st.code, st.text, st.latext, st.write]
"""

human_template = """
args: next_segment
data type: string
instruction: Display the generated next narrative segment to the user
code:
if next_segment is not None and len(str(next_segment)) > 0:
    st.success(next_segment)

args:{args}
data type: {data_type}
instruction:{instruction}
code:

"""

code = "st.markdown({args})"


system_template = f"""
You are a system architect that can determine system inputs of the architecture that the project needs even if it is not explicity mentioned in the Project Decription.
You will get Project Decription from client then generate list of system inputs.
This list includes both the initial and intermediate inputs that the system should take to work properly
If the system is chat-based then, it should have an input corresponding to that as one of the system inputs such as message.
It should be a valid Python list
"""

human_template = """
Project Decription: Application that can analyze the user
System Inputs: ["answer"]

Project Decription: Create a system that can summarize a powerpoint file
System Inputs:["powerpoint_file"]

Project Decription: Create a Bill Gates clone
System Inputs:["message"]

Project Decription: Generate a system that enable me to give the teacher field then make a chat with the teacher.
System Inputs:["message", "teacher_field"]

Project Decription: Create a translator which translates to any language
System Inputs:["output_language", "source_text"]

Project Decription: Create an app that I can chat with
System Inputs:["message"]

Project Decription: Generate a system that can generate tweet from hashtags and give a score for the tweet.
System Inputs:["hashtags"]

Project Decription: Generate a chat-based system that can analyze the given csv file.
System Inputs:["message", "csv_file"]

Project Decription: Summarize a text taken from the user
System Inputs:["text"]

Project Decription: Create a platform which lets the user select a lecture and then show topics for that lecture 
then give a question to the user. After user gives his/her answer, it gives a score for the answer and give explanation.
System Inputs:["lecture", "topic", "user_answer"]

Project Decription: Create a system that can generate blog post related to a website
System Inputs: ["url"]

Project Decription: {instruction}
System Inputs:
"""



system_template = """
You will write a single line streamlit input code such as st.text_input, st.selectbox...
You will do it to accomplish the given instruction. You will see examples. You will also see the previous code
segment that you will continue on. You will only write a single streamlit code by looking both the instruction and the previous code
"""

human_template = """
Previous Code Segment:{code_snippets}

variable: url
instruction: Get website URL from the user
code:
url = st.text_input("Enter website URL")

variable: source_text
instruction: Get source text from the user
code:
source_text = st.text_area("Enter source text")

variable: input_language
instruction: Get the input language from the user
code:
input_language = st.text_input("Enter the input language")

variable: color
instruction: Select the color
code:
color = st.selectbox("Select the color", ["Red", "Blue", "Green", "Yellow", "purple"])

variable: {variable}
instruction: {instruction}
code:
"""



system_template = """
You are an AI assistant that write a concise prompt to direct an assistant to make web search for the given instruction.
You will have inputs and instruction. The prompt should be formattable with the inputs which means it should include inputs with curly braces.
"""

human_template = """
Instruction: Search the given input
Inputs:input
Prompt: Find the answer of it: {{input}}

Instruction: Find the list of song releated to the title
Inputs:title
Prompt: Find the list of songs releated to the title: {{title}}

Instruction:{instruction}
Inputs:{inputs}
Prompt:
"""

imports = """
from langchain.agents import ConversationalChatAgent, AgentExecutor
from langchain.tools import DuckDuckGoSearchRun
from langchain.memory.chat_message_histories import StreamlitChatMessageHistory
from langchain.memory import ConversationBufferMemory
from langchain.agents.tools import Tool
from langchain.chains import LLMMathChain
from langchain.chat_models import ChatOpenAI
from langchain.callbacks import StreamlitCallbackHandler

msgs = StreamlitChatMessageHistory()
memory = ConversationBufferMemory(
    chat_memory=msgs, return_messages=True, memory_key="chat_history", output_key="output"
)
"""

functions = """
def {function_name}({argument}):
    llm = ChatOpenAI(model_name="gpt-3.5-turbo-16k", openai_api_key=openai_api_key)
    llm_math_chain = LLMMathChain.from_llm(llm=llm, verbose=True)
    tools = [
        DuckDuckGoSearchRun(name="Search"),
        Tool(
            name="Calculator",
            func=llm_math_chain.run,
            description="useful for when you need to answer questions about math"
        )]
    chat_agent = ConversationalChatAgent.from_llm_and_tools(llm=llm, tools=tools)
    executor = AgentExecutor.from_agent_and_tools(
        agent=chat_agent,
        tools=tools,
        memory=memory,
        return_intermediate_steps=True,
        handle_parsing_errors=True,
    )
    st_cb = StreamlitCallbackHandler(st.container(), expand_new_thoughts=False)
    return executor({argument}, callbacks=[st_cb])["output"]
"""

outputs = """
if not openai_api_key.startswith('sk-'):
    st.warning('Please enter your OpenAI API key!', icon='⚠')
    {variable} = ""
elif {argument}:
    {variable} = {function_name}({argument})
else:
    {variable} = ''
"""


INSTRUCTIONS = [
    "generate a system that reads uploaded text file and translates its content into the language that user prompted",
    "Create a system that can translate from any language to any language",
    "Create a system that can generate blog post related to a website then summarize the generated blog post and show only the summarization",
    "create lyrics from a song title",
    "Create a system that answers question related to the uploaded pdf.",
    "Create an application that gets csv file as an input then shows the summarization of that file.",
    "Create a system that generates random programming related humors when 'laugh' button is clicked without any user input by AI",
]

CODE_SNIPPETS = [
    {
        "instruction": "generate a system that reads uploaded text file and translates its content into the language that user prompted",
        "plan": """
1. Get the file path from the user by 'ui_input_file'
2. Use 'doc_loader' to load the text file as Document from the file path.
3. Use 'doc_to_string' to convert Document to string
4. Get the output language from the user by 'ui_input_text'
5. If all inputs are filled, use 'prompt_template' to translate the text to the output language.
6. If the translation is ready, display it to the user by 'ui_output_text'.
""",
        "code_snippets": """
#Get the file path from the user
file = st.file_uploader("Upload file", type=["txt", "pdf", "docx"])
if file is not None:
    file_path = file.name
    st.session_state['file_path'] = file_path
else:
    st.warning("Please upload a file.")
    
# Return the file path as a string
if 'file_path' in st.session_state:
    file_path = st.session_state['file_path']
    st.write(f"File path: {file_path}")
    st.write(f"Type: {type(file_path).__name__}")



#Load the text file as Document from the file path
from langchain.document_loaders import TextLoader

def load_document(file_path):
    loader = TextLoader(file_path)
    docs = loader.load()
    return docs



#Convert Document to string
text = str(document)



#Get the output language from the user
output_language = st.text_input("Enter the output language:")



#Translate the text to the output language
def translator(text,output_language):
    chat = ChatOpenAI(
        temperature=0
    )
    system_template = "You are a language translator. Your task is to translate text to {output_language}."
    system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
    human_template = "Please translate the following text to {output_language}: '{text}'."
    human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
    chat_prompt = ChatPromptTemplate.from_messages(
        [system_message_prompt, human_message_prompt]
    )

    chain = LLMChain(llm=chat, prompt=chat_prompt)
    result = chain.run(text=text, output_language=output_language)
    return result # returns string   


if text and output_language:
    translation = translator(text,output_language)
else:
    translation = ""



#Display the translated text to the user
if translation:
    st.markdown(f"Translated Text: {translation}")
""",
    },
    {
        "instruction": "Create a system that can generate blog post related to a website then summarize it",
        "plan": """
1. Get website URL from the user by 'ui_input_text'
2. Use 'doc_loader' to load the website as Document from URL
3. Use 'doc_to_string' to convert Document to string
4. If doc_to_string generated the content as string, use 'prompt_template' to generate a blog post related to that content.
5. If blog post is generated, use 'prompt_template' to summarize the blog post.
6. If summarization is ready, display it to the user by 'ui_output_text'.
""",
        "code_snippets": """
#Get website url from the user
url = st.text_input('Enter website URL:')

#Load the document from the website url
from langchain.document_loaders import WebBaseLoader

def doc_loader(url):
    loader = WebBaseLoader(url)
    docs = loader.load()
    return docs

#Convert docs to string
docs_string = str(docs)

#Write blog post related to the context of docs_string
def blogPostWriter(docs_string):
    chat = ChatOpenAI(
        temperature=0.7
    )
    system_template = "You are a blogger tasked with writing a blog post related to the following topic: '{docs_string}'."
    system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
    human_template = "Please write a blog post related to the following topic: '{docs_string}'."
    human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
    chat_prompt = ChatPromptTemplate.from_messages(
        [system_message_prompt, human_message_prompt]
    )

    chain = LLMChain(llm=chat, prompt=chat_prompt)
    result = chain.run(docs_string=docs_string)
    return result # returns string   


if docs_string:
    blog = blogPostWriter(docs_string)
else:
    blog = ""

#Summarize the blog post
def blogSummarizer(blog):
    chat = ChatOpenAI(
        temperature=0
    )
    system_template = "You are an AI assistant tasked with summarizing a blog post."
    system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
    human_template = "Please summarize the following blog post: '{blog}'."
    human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
    chat_prompt = ChatPromptTemplate.from_messages(
        [system_message_prompt, human_message_prompt]
    )

    chain = LLMChain(llm=chat, prompt=chat_prompt)
    result = chain.run(blog=blog)
    return result # returns string   


if blog:
    summarization = blogSummarizer(blog)
else:
    summarization = ""

#Display the generated summarization to the user
def show_summarization(summarization):
    if summarization:
        st.markdown(f"## Summarization:\n{summarization}")
    else:
        st.markdown("Please enter a valid input to generate a summarization.")

show_summarization(summarization)
    """,
    },
]

TEST_CASES = [
    {
        "instruction": "Create a system that can generate blog post related to a website then summarize it"
    },
    {
        "instruction": """
        create a system that can predict horoscope by asking intelligent question 
        to the user and analyzing user's answer without birth date or explicit question directly related to horoscope.
        """,
        "plan": """
        1. Generate intelligent questions related to horoscope using AI.
        2. Show the question to the user.
        3. Get answer from the user for the asked question.
        4. Analyze user's answer using AI to predict horoscope.
        5. Show the horoscope prediction to the user.
        """,
        "task": """
        [
            {
                "step": 1,
                "task_type": "prompt_template",
                "task_name": "generate_intelligent_questions",
                "input_key": "none",
                "output_key": "generated_questions",
                "description": "Generate intelligent questions related to horoscope using AI."
            },
            {
                "step": 2,
                "task_type": "ui_output_text",
                "task_name": "show_question_to_user",
                "input_key": "generated_questions",
                "output_key": "none",
                "description": "Show the question to the user."
            },
            {
                "step": 3,
                "task_type": "ui_input_text",
                "task_name": "get_user_answer",
                "input_key": "none",
                "output_key": "user_answer",
                "description": "Get answer from the user for the asked question."
            },
            {
                "step": 4,
                "task_type": "prompt_template",
                "task_name": "analyze_user_answer",
                "input_key": "user_answer, context",
                "output_key": "horoscope_prediction",
                "description": "Analyze user's answer using AI to predict horoscope."
            },
            {
                "step": 5,
                "task_type": "ui_output_text",
                "task_name": "show_horoscope_prediction",
                "input_key": "horoscope_prediction",
                "output_key": "none",
                "description": "Show the horoscope prediction to the user."
            }
        ]
        """,
        "code_snippets": """
        import streamlit as st
        from langchain.chains import LLMChain
        from langchain.chat_models import ChatOpenAI
        from langchain.prompts.chat import (ChatPromptTemplate,
                                            HumanMessagePromptTemplate,
                                            SystemMessagePromptTemplate)


        st.title(My App)
        def horoscopeQuestionGenerator():
            chat = ChatOpenAI(
                temperature=0.7
            )
            system_template = "You are an AI assistant designed to generate intelligent questions related to horoscopes."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "Please generate an intelligent question related to horoscopes."
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run({})
            return result # returns string   

        generated_questions = horoscopeQuestionGenerator()
        def show_question(generated_questions):
            if generated_questions != "":
                st.markdown("Question: " + generated_questions)

        show_question(generated_questions)
        user_answer = st.text_input("Enter your answer")
        def horoscopePredictor(user_answer,context):
            chat = ChatOpenAI(
                temperature=0
            )
            system_template = "You are skilled at predicting horoscopes based on analyzed traits and characteristics."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "The user's answer is: {user_answer}. The context is: {context}. Please predict their horoscope based on this information."
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run(user_answer=user_answer, context=context)
            return result # returns string   

        horoscope_prediction = horoscopePredictor(user_answer,context)
        import streamlit as st

        def show_horoscope_prediction(horoscope_prediction):
            if horoscope_prediction != "":
                st.markdown("## Horoscope Prediction")
                st.markdown(horoscope_prediction)

        show_horoscope_prediction(horoscope_prediction)
        """,
    },
    {
        "instruction": """
        create a system that can translate a text to any determined language by the user
        """,
        "plan": """
        1. Get the source text from the user.
        2. Get the desired output language from the user.
        3. If both inputs are filled, use AI to translate the text to the output language.
        4. If the translation is ready, return it to the user.
        """,
        "task": """
        [
            {
                "step": 1,
                "task_type": "ui_input_text",
                "task_name": "get_source_text",
                "input_key": "none",
                "output_key": "source_text",
                "description": "Get the source text from the user."
            },
            {
                "step": 2,
                "task_type": "ui_input_text",
                "task_name": "get_output_language",
                "input_key": "none",
                "output_key": "output_language",
                "description": "Get the desired output language from the user."
            },
            {
                "step": 3,
                "task_type": "prompt_template",
                "task_name": "translate_text",
                "input_key": [
                    "source_text",
                    "output_language"
                ],
                "output_key": "translated_text",
                "description": "Use AI to translate the text to the output language."
            },
            {
                "step": 4,
                "task_type": "ui_output_text",
                "task_name": "display_translation",
                "input_key": "translated_text",
                "output_key": "none",
                "description": "Return the translated text to the user."
            }
        ]
        """,
        "code_snippets": """
        import streamlit as st
        from langchain.chains import LLMChain
        from langchain.chat_models import ChatOpenAI
        from langchain.prompts.chat import (ChatPromptTemplate,
                                            HumanMessagePromptTemplate,
                                            SystemMessagePromptTemplate)


        st.title(My App)
        source_text = st.text_area("Enter the source text")
        button = st.button("Submit")
        output_language = st.text_input("Enter desired output language")
        button = st.button("Submit")
        def languageTranslator(source_text,output_language):
            chat = ChatOpenAI(
                temperature=0
            )
            system_template = "You are an AI language translator. Your task is to translate text to {output_language}."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "Please translate the following text to {output_language}: '{source_text}'."
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run(source_text=source_text, output_language=output_language)
            return result # returns string   

        translated_text = languageTranslator(source_text,output_language)
        def show_translated_text(translated_text):
            if translated_text != "":
                st.markdown("Translated Text: " + translated_text)

        show_translated_text(translated_text)
        """,
    },
    {
        "instruction": """
        create lyrics from a song title
        """,
        "plan": """
        1. Get song title from the user.
        3. Use AI to generate lyrics appropriate for the song title.
        4. If lyrics is ready, display it to the user.
        """,
        "task": """
        [
            {
                "step": 1,
                "task_type": "ui_input_text",
                "task_name": "get_song_title",
                "input_key": "none",
                "output_key": "song_title",
                "description": "Gets song title from the user."
            },
            {
                "step": 2,
                "task_type": "prompt_template",
                "task_name": "generate_lyrics",
                "input_key": "song_title",
                "output_key": "lyrics",
                "description": "Use AI to generate lyrics for the song."
            },
            {
                "step": 3,
                "task_type": "ui_output_text",
                "task_name": "return_song",
                "input_key": "lyrics",
                "output_key": "none",
                "description": "If lyrics is ready, return it to the user."
            }
        ]
        """,
        "code_snippets": """
        import streamlit as st
        from langchain.chains import LLMChain
        from langchain.chat_models import ChatOpenAI
        from langchain.prompts.chat import (ChatPromptTemplate,
                                            HumanMessagePromptTemplate,
                                            SystemMessagePromptTemplate)


        st.title(My App)
        song_title = st.text_input("Enter song title")
        button = st.button("Submit")
        def melodyGenerator(song_title):
            chat = ChatOpenAI(
                temperature=0.7
            )
            system_template = "You are an AI music composer. Your task is to generate a melody for the song '{song_title}'."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "Please use AI to generate a melody for the song '{song_title}'."
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run(song_title=song_title)
            return result # returns string   

        melody = melodyGenerator(song_title)
        def lyricGenerator(song_title):
            chat = ChatOpenAI(
                temperature=0.7
            )
            system_template = "You are an AI songwriter. Your task is to generate lyrics for a song with the title: '{song_title}'."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "Please generate lyrics for a song titled '{song_title}'."
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run(song_title=song_title)
            return result # returns string   

        lyrics = lyricGenerator(song_title)
        import streamlit as st

        def show_text(melody, lyrics):
            if melody != "" and lyrics != "":
                st.markdown("Melody: {}".format(melody))
                st.markdown("Lyrics: {}".format(lyrics))
            else:
                st.markdown("Please provide both melody and lyrics.")

        if melody != "" and lyrics != "":
            show_text(melody, lyrics)
        """,
    },
    {
        "instruction": """
        generate a system that reads uploaded text file and translates its content into the language that user prompted
        """,
        "plan": """
        1. Get the input language from the user.
        2. Get the output language from the user.
        3. Get the text file from the user.
        4. Read the content of the text file.
        5. Use AI to translate the content from the input language to the output language.
        6. If the translation is successful, return the translated content to the user.
        """,
        "task": """
        [
            {
                "step": 1,
                "task_type": "ui_input_text",
                "task_name": "get_input_language",
                "input_key": "none",
                "output_key": "input_language",
                "description": "Get the input language from the user."
            },
            {
                "step": 2,
                "task_type": "ui_input_text",
                "task_name": "get_output_language",
                "input_key": "none",
                "output_key": "output_language",
                "description": "Get the output language from the user."
            },
            {
                "step": 3,
                "task_type": "ui_input_file",
                "task_name": "get_text_file",
                "input_key": "none",
                "output_key": "text_file",
                "description": "Get the text file from the user."
            },
            {
                "step": 4,
                "task_type": "ui_output_text",
                "task_name": "read_text_file",
                "input_key": "text_file",
                "output_key": "file_content",
                "description": "Read the content of the text file."
            },
            {
                "step": 5,
                "task_type": "prompt_template",
                "task_name": "translate_content",
                "input_key": "file_content, input_language, output_language",
                "output_key": "translated_content",
                "description": "Use AI to translate the content from the input language to the output language."
            },
            {
                "step": 6,
                "task_type": "ui_output_text",
                "task_name": "return_translated_content",
                "input_key": "translated_content",
                "output_key": "none",
                "description": "If the translation is successful, return the translated content to the user."
            }
        ]
        """,
        "code_snippets": """
        import streamlit as st
        from langchain.chains import LLMChain
        from langchain.chat_models import ChatOpenAI
        from langchain.prompts.chat import (ChatPromptTemplate,
                                            HumanMessagePromptTemplate,
                                            SystemMessagePromptTemplate)


        st.title(My App)
        input_language = st.text_input("Enter the input language")
        output_language = st.text_input("Enter the output language")
        if st.button("Submit"):
            # Perform some action with the output_language variable
        uploaded_file = st.file_uploader("Choose a file", type="txt")
        if uploaded_file is not None:
            text_file = uploaded_file.read().decode('utf-8')
            st.write(text_file)
        import streamlit as st

        def show_text(text_file):
            if text_file != "":
                with open(text_file, "r") as file:
                    text = file.read()
                st.markdown(text)

        show_text(text_file)
        def languageTranslator(file_content,input_language,output_language):
            chat = ChatOpenAI(
                temperature=0
            )
            system_template = "You are an AI language translator. Your task is to translate content from {input_language} to {output_language}."
            system_message_prompt = SystemMessagePromptTemplate.from_template(system_template)
            human_template = "Translate the following content from {input_language} to {output_language}:

        {file_content}"
            human_message_prompt = HumanMessagePromptTemplate.from_template(human_template)
            chat_prompt = ChatPromptTemplate.from_messages(
                [system_message_prompt, human_message_prompt]
            )

            chain = LLMChain(llm=chat, prompt=chat_prompt)
            result = chain.run(file_content=file_content, input_language=input_language, output_language=output_language)
            return result # returns string   

        translated_content = languageTranslator(file_content,input_language,output_language)
        import streamlit as st

        def show_text(translated_content):
            if translated_content != "":
                st.markdown(translated_content)

        show_text(translated_content)
        """,
    },
]

TOOL_EXAMPLES = {
    "ui_input_text": [
        {
            "instruction": "Get answer from the user for the asked question",
            "variable": "answer",
        },
        {"instruction": "Get song title from the user", "variable": "song_title"},
        {"instruction": "Get the source text from the user", "variable": "source_text"},
    ],
    "ui_output_text": [
        {
            "instruction": "Show the predicted horoscope to the user",
            "args": "horoscope",
        },
        {
            "instruction": "Return the translated text to the user",
            "args": "translated_text",
        },
        {"instruction": "Show the generated questions to the user", "args": "question"},
    ],
    "prompt_template": [
        {
            "instruction": "Generate intelligent questions related to horoscope",
            "inputs": "user",
        },
        {
            "instruction": "Analyze user's answers using AI to predict their horoscope",
            "inputs": "answer",
        },
        {
            "instruction": "Translate the text to the output language using AI",
            "inputs": """[
                "source_text",
                "output_language"
            ]""",
        },
    ],
}



code = """
with st.chat_message("assistant"):
    message_placeholder = st.empty()
    full_response = ""
    # Simulate stream of response with milliseconds delay
    for chunk in {res}.split():
        full_response += chunk + " "
        time.sleep(0.05)
        # Add a blinking cursor to simulate typing
        message_placeholder.markdown(full_response + "▌")
    message_placeholder.markdown(full_response)
    # Add assistant response to chat history
    if full_response:
        st.session_state.messages.append({{"role": "assistant", "content": full_response}})        
"""


# from .task_definitions import TASK_DESCRIPTIONS, TASK_NAMES

system_template = """
Create a Python list of task objects that align with the provided instruction and plan. Task objects must be Python dictionaries, and the output should strictly conform to a Python list of JSON objects.

You must use only the tasks provided in the description:

{TASK_DESCRIPTIONS}

task_name could be only one of the task names below:
{TASK_NAMES}
"""

human_template = """
Create a Python list of task objects that align with the provided instruction and all steps of the plan.

Task objects must be Python dictionaries, and the output should strictly conform to a Python list of JSON objects.

Follow these detailed guidelines:

Task Objects: Create a Python dictionary for each task using the following keys:

step: It represents the step number corresponding to which plan step it matches
task_type: Should match one of the task names provided in task descriptions.
task_name: Define a specific name for the task that aligns with the corresponding plan step.
input_key: List the "output_key" values from parent tasks used as input or "none" if there's no input or if it comes from the user.
input_data_type: The list of data types of the inputs
output_key: Designate a unique key for the task's output. It is compatible with the output type if not none
output_data_type: The data type of the output
description: Provide a brief description of the task's goal, mirroring the plan step.

Ensure that each task corresponds to each step in the plan, and that no step in the plan is omitted.
Ensure that output_key is unique for each task.
Ensure that each task corresponds to each step in the plan
Ensure that an output type of task does not change.

##########################
Instruction: Create a system that can generate blog post related to a website
Plan:
1. Get website URL from the user with 'ui_input_text'
2. Use 'doc_loader' to load the page as Document
3. Use 'doc_to_string' to convert Document to string
4. Use 'prompt_template' to generate a blog post using the result of doc_to_string
5. If blog post is generated, show it to the user with 'ui_output_text'.
List of Task Objects (Python List of JSON):
[
    {{
        "step": 1,
        "task_type": "ui_input_text",
        "task_name": "get_url",
        "input_key": "none",
        "input_data_type": "none",
        "output_key": "url",
        "output_data_type": "string",
        "description": "Get website url from the user"
    }},
    {{
        "step": 2,
        "task_type": "doc_loader",
        "task_name": "doc_loader",
        "input_key": "url",
        "input_data_type": "string",
        "output_key": "docs",
        "output_data_type": "Document",
        "description": "Load the document from the website url"
    }},
    {{
        "step": 3,
        "task_type": "doc_to_string",
        "task_name": "convertDocToString",
        "input_key": "docs",
        "input_data_type": "Document",
        "output_key": "docs_string",
        "output_data_type": "string",
        "description": "Convert docs to string"
    }},
    {{
        "step": 4,
        "task_type": "prompt_template",
        "task_name": "writeBlogPost",
        "input_key": ["docs_string"],
        "input_data_type": ["string"],
        "output_key": "blog",
        "output_data_type": "string",
        "description": "Write blog post related to the context of docs_string"
    }},
    {{
        "step": 5,
        "task_type": "ui_output_text",
        "task_name": "show_blog",
        "input_key": "blog",
        "input_data_type": "string",
        "output_key": "none",
        "output_data_type": "none",
        "description": "Display the generated blog post to the user"
    }}
]
##########################
Instruction:{instruction}
Plan : {plan}
List of Task Objects (Python List of JSON):
"""



loaders = """
For Local TXT file:
TextLoader
################################
For Web Page:
WebBaseLoader
################################
For Online PDF:
OnlinePDFLoader
################################
For Local PDF:
UnstructuredPDFLoader
################################
For Power Point:
UnstructuredPowerPointLoader
################################
For CSV:
CSVLoader
################################
For Excel:
UnstructuredExcelLoader
################################
For Docx:
UnstructuredWordDocumentLoader
################################
For Youtube:
YoutubeLoader
################################
For Notion Zip File:
NotionDirectoryLoader
"""

system_template = f"""
Based on the provided context in 'Previous Code', choose the most appropriate loader.

These are your loader options:

{loaders}
"""

human_template = """
Use the information from 'Previous Code' to determine the loader from one of the loader options.
Don't write any explanation but directly say the loader option

Instruction: {instruction}  
Previous Code: {code_snippets}
Loader Option:
"""

imports = """
import shutil
from langchain.document_loaders import *

"""

functions = """

def {function_name}({argument}):
    {loader_line}
    docs = loader.load()
    return docs
"""

outputs = """
if {argument}:
    {variable} = {function_name}({argument})
else:
    {variable} = ''
"""


system_template = """
Refine the Generated Task List by fixing the problem mentinoed in the Feedback

Task objects must be Python dictionaries, and the output should strictly conform to a Python list of JSON objects.
So use double quotes because this output will be converted to json with json.loads function.

It is extremely important to include these keys in each task object:

-step
-task_type
-task_name
-input_key
-input_data_type
-output_key
-output_data_type
-description

################################

You are only allowed to use those tasks below:

{TASK_NAMES}

These are the explanations of those tasks:

{TASK_PURPOSES}

Your main job is by considering the "Problems", generating new task list
Please ensure that the New Refined Task List does not contain any problem mentioned in the "Problems".
If needed, you can change the number of tasks, remove/replace/add tasks as long as you use only the allowed tasks
"""

human_template = """
Instruction:{instruction}
################################
Problematic Task List:
{tasks}
################################
Problems: {feedback}
################################
New Refined Task List:
"""



import json
import os
import re
from difflib import SequenceMatcher

from langchain.chains import LLMChain
from langchain.chat_models import ChatOpenAI
from langchain.prompts.chat import (ChatPromptTemplate,
                                    HumanMessagePromptTemplate,
                                    SystemMessagePromptTemplate)

from demogpt import utils
from demogpt.chains import prompts


class TaskChains:
    llm = None

    @classmethod
    def setLlm(
        cls,
        model,
        openai_api_key=os.getenv("OPENAI_API_KEY", ""),
        temperature=0.0,
        openai_api_base=None,
    ):
        cls.llm = ChatOpenAI(
            model=model,
            openai_api_key=openai_api_key,
            temperature=temperature,
            openai_api_base=openai_api_base
        )

    @classmethod
    def getChain(cls, system_template="", human_template="", **kwargs):
        prompts = []
        if system_template:
            prompts.append(SystemMessagePromptTemplate.from_template(system_template))
        if human_template:
            prompts.append(HumanMessagePromptTemplate.from_template(human_template))
        chat_prompt = ChatPromptTemplate.from_messages(prompts)
        return LLMChain(llm=cls.llm, prompt=chat_prompt).run(**kwargs)

    @classmethod
    def uiInputText(cls, task):
        variable = ", ".join(task["output_key"])
        instruction = task["description"]
        code = cls.getChain(
            human_template=prompts.ui_input_text.human_template,
            instruction=instruction,
            variable=variable,
        )
        return utils.refine(code)

    @classmethod
    def uiOutputText(cls, task):
        args = ", ".join(task["input_key"])
        data_type = task["input_data_type"]
        if isinstance(args, list):
            args = ",".join(args)
        instruction = task["description"]
        code = cls.getChain(
            system_template=prompts.ui_output_text.system_template,
            human_template=prompts.ui_output_text.human_template,
            instruction=instruction,
            args=args,
            data_type=data_type,
        )
        return utils.refine(code)

    @classmethod
    def uiInputFile(cls, task):
        variable = ", ".join(task["output_key"])
        instruction = task["description"]
        res = cls.getChain(
            system_template=prompts.ui_input_file.system_template,
            human_template=prompts.ui_input_file.human_template,
            instruction=instruction
        )
        res = res[res.find("{") : res.rfind("}") + 1]
        res = json.loads(res)
        title = res.get("title")
        data_type = res.get("data_type")
        code = prompts.ui_input_file.code.format(instruction=instruction,
                                                 title=title,
                                                 data_type=data_type,
                                                 variable=variable
                                                 )
        return code

    @classmethod
    def pathToContent(cls, task, code_snippets):
        instruction = task["description"]
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])

        code = cls.getChain(
            system_template=prompts.path_to_file.system_template,
            human_template=prompts.path_to_file.human_template,
            instruction=instruction,
            argument=argument,
            variable=variable,
            code_snippets=code_snippets,
        )
        return utils.refine(code)
    
    @classmethod
    def getDetailedDescription(cls, app_idea, instruction):
        return cls.getChain(
            system_template=prompts.detailed_description.system_template,
            human_template=prompts.detailed_description.human_template,
            app_idea=app_idea,
            instruction=instruction
        )

    @classmethod
    def promptTemplate(cls, app_idea, task):
        inputs = ", ".join(task["input_key"])
        instruction = task["description"]

        res = cls.getChain(
            system_template=prompts.prompt_template.system_template,
            human_template=prompts.prompt_template.human_template,
            instruction=instruction,
            inputs=inputs,
        )
        res = res[res.find("{") : res.rfind("}") + 1]
        return json.loads(res)

    @classmethod
    def uiInputChat(cls, task):
        variable = ", ".join(task["output_key"])
        instruction = task["description"]

        placeholder = cls.getChain(
            human_template=prompts.ui_input_chat.human_template,
            instruction=instruction,
            variable=variable,
        )
        
        code = prompts.ui_input_chat.code.format(variable=variable, placeholder=placeholder)
        
        return code

    @classmethod
    def uiOutputChat(cls, task):
        res = ", ".join(task["input_key"])

        code = prompts.ui_output_chat.code.format(res=res)
        
        return code

    @classmethod
    def chat(cls, app_idea, task):
        inputs = ", ".join(task["input_key"])
        instruction = task["description"]
        
        new_instruction = cls.getDetailedDescription(app_idea=app_idea,instruction=instruction)

        res = cls.getChain(
            system_template=prompts.chat.system_template,
            human_template=prompts.chat.human_template,
            instruction=new_instruction,
            inputs=inputs,
        )
        res = res.replace("'''", '"""')
        res = res[res.find("{") : res.rfind("}") + 1]
        return json.loads(res, strict=False)

    @classmethod
    def promptTemplateRefiner(cls, templates, inputs, feedback):
        res = cls.getChain(
            system_template=prompts.prompt_chat_refiner.system_template,
            human_template=prompts.prompt_chat_refiner.human_template,
            templates=templates,
            feedback=feedback,
            inputs=inputs,
        )
        res = res[res.find("{") : res.rfind("}") + 1]
        return json.loads(res)
    
    @classmethod
    def search_chat(cls, task):
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        function_name = task["task_name"]
        instruction = task["description"]

        res = cls.getChain(
            system_template=prompts.search_chat.system_template,
            human_template=prompts.search_chat.human_template,
            instruction=instruction,
            inputs=argument,
        )
        
        res = res.replace('"',"'")

        imports = prompts.search_chat.imports
        functions = prompts.search_chat.functions.format(function_name=function_name, argument=argument)
        outputs = prompts.search_chat.outputs.format(function_name=function_name, argument=argument,variable=variable)
        
        code = imports + "\n" + functions + "\n" + outputs + "\n"
        
        return code

    @classmethod
    def search(cls, task):
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        function_name = task["task_name"]
        instruction = task["description"]

        res = cls.getChain(
            system_template=prompts.search.system_template,
            human_template=prompts.search.human_template,
            instruction=instruction,
            inputs=argument,
        )
        
        res = res.replace('"',"'")

        imports = prompts.search.imports
        functions = prompts.search.functions.format(function_name=function_name, argument=argument,res=res)
        outputs = prompts.search_chat.outputs.format(function_name=function_name, argument=argument,variable=variable)
        code = imports + "\n" + functions + "\n" + outputs + "\n"
        
        return code

    @classmethod
    def docLoad(cls, task, code_snippets):
        
        def get_most_similar_key(input_key, available_keys):
            # This function returns the most similar key from available_keys to the input_key.
            best_match = None
            best_ratio = 0
            for key in available_keys:
                ratio = SequenceMatcher(None, input_key, key).ratio()
                if ratio > best_ratio:
                    best_ratio = ratio
                    best_match = key
            return best_match
        
        type2loader = {
            "txt": "TextLoader",
            "docx":"UnstructuredWordDocumentLoader",
            "pdf":"UnstructuredPDFLoader",
            "pptx":"UnstructuredPowerPointLoader",
            "csv":"CSVLoader",
            "xlsx":"UnstructuredExcelLoader",
            "zip":"NotionDirectoryLoader",
            "online_pdf":"OnlinePDFLoader",
            "web":"WebBaseLoader",
            "xlsx":"UnstructuredExcelLoader",
            "youtube":"YoutubeLoader",  
        }
        
        loader2type = {type2loader[dtype]:dtype for dtype in type2loader}
        
        def getLoaderCall(data_type):
            if data_type in loader2type:
                data_type = loader2type[data_type]
            
            loader = type2loader.get(data_type)  # First, try to get the exact match
            if loader is None:
                # If there's no exact match, get the most similar key and retrieve the value
                similar_key = get_most_similar_key(data_type, type2loader.keys())
                loader = type2loader[similar_key]
                
            loader = type2loader[data_type]
            
            if data_type in [
                "txt",
                "online_pdf",
                "docx",
                "csv"
            ]:
                loader_line = f"loader = {loader}({argument})"
            elif data_type == "web":
                loader_line = f"loader = {loader}([{argument}])"
            elif data_type in ["pdf", "pptx"]:
                loader_line = (
                    f'loader = {loader}({argument}, mode="elements", strategy="fast")'
                )
            elif data_type  == "xlsx":
                loader_line = f'loader = {loader}({argument}, mode="elements")'
            elif data_type == "youtube":
                loader_line = (
                    f"loader = {loader}.from_youtube_url({argument}, add_video_info=False)"
                )
            elif data_type == "zip":
                loader_line = f"""if os.path.exists('Notion_DB') and os.path.isdir('Notion_DB'):
            shutil.rmtree('Notion_DB')
        os.system(f"unzip {{{argument}}} -d Notion_DB")
        loader = {loader}("Notion_DB")"""
            else:
                loader_line = f"loader = TextLoader({argument})"
            
            return loader_line
        
        instruction = task["description"]
        argument = task["input_key"][0]
        variable = ", ".join(task["output_key"])
        function_name = task["task_name"]
        
        variable_match = re.search(r"(\w+)\s*=\s*temp_file\.name", code_snippets, re.MULTILINE)
        
        if variable_match:
            variable_name = variable_match.group(1).strip()
        else:
            variable_name = ''
            
        match = re.search(r"st\.file_uploader\(\s*?.*?type=\s*\[(.*?)\]\s*?.*?\)", code_snippets, re.DOTALL)
                
        loader_line = ""

        if variable_name == argument and  match:
            types = match.group(1).replace("'", "").split(", ")
            types = [type.strip() for type in types]
            if len(types) == 1:
                loader_line = getLoaderCall(types[0])
            else:
                loader_line = "\n    ".join([f"if {argument}.endswith('.{data_type}'):\n\t{getLoaderCall(data_type)}" for data_type in types])
        else:
            print("No match found. Using the chain...")
            types = []
            loader = cls.getChain(
                system_template=prompts.doc_load.system_template,
                human_template=prompts.doc_load.human_template,
                instruction=instruction,
                code_snippets=code_snippets
                )
            
            loader_line = getLoaderCall(loader)
            
        imports = prompts.doc_load.imports
        functions = prompts.doc_load.functions.format(function_name=function_name, argument=argument, loader_line=loader_line)
        outputs = prompts.doc_load.outputs.format(argument=argument, function_name=function_name, variable=variable)
        
        code = imports + "\n" + functions + "\n" + outputs + "\n"
        
        return code

    @classmethod
    def stringToDoc(cls, task):
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        imports = prompts.string_to_doc.imports
        outputs = prompts.string_to_doc.outputs.format(variable=variable, argument=argument)
        code = imports + "\n" + outputs + "\n"
        
        return code

    @classmethod
    def docToString(cls, task):
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        code = f'{variable} = "".join([doc.page_content for doc in {argument}])'
        return code

    @classmethod
    def summarize(cls, task):
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        function_name = task["task_name"]

        imports = prompts.summarize.imports
        functions = functions = prompts.summarize.functions.format(function_name=function_name, argument=argument)
        outputs = prompts.string_to_doc.outputs.format(function_name=function_name, variable=variable, argument=argument)
        
        code = imports + "\n" + functions + "\n" + outputs + "\n"
        
        return code

    @classmethod
    def pythonCoder(cls, task, code_snippets):
        instruction = task["description"]
        argument = ", ".join(task["input_key"])
        variable = ", ".join(task["output_key"])
        function_name = task["task_name"]

        code = cls.getChain(
            system_template=prompts.python_coder.system_template,
            human_template=prompts.python_coder.human_template,
            instruction=instruction,
            argument=argument,
            variable=variable,
            function_name=function_name,
            code_snippets=code_snippets,
        )
        return utils.refine(code)


system_template = """
You are a helpful assistant that can transform the given directive.
This directive is given to the ai-based question answering system. However, it is possible that the directive misses the functionality of the instruction.
You task is refine the directive in a way that it tells the functionality of the instruction so that the syetem knows how to behave.
You can see the real functionality of the model by looking at the Instruction.
Please generate a 1 sentence long directive. Use the Original Directive's style while generating the Refined Directive.
"""

human_template = """
Instruction:{app_idea}

Original Directive:{instruction}

Refined Directive:
"""


import json

AVAILABLE_TASKS_COUNT = 14

################################

ALL_TASKS = [
    {
        "name": "ui_input_text",
        "description": "Gets input from the user via a text field.",
        "good_at": "Retrieving text input from the user.",
        "input_data_type": "none",
        "output_data_type": "string",
        "purpose": "Collect user-entered text for further processing.",
    },
    {
        "name": "ui_input_file",
        "description": "Provide a mechanism for users to upload a file and return its path. The task involves creating a file upload widget and returning its file path",
        "good_at": "Enabling file uploads and making the file path available for doc_load",
        "input_data_type": "none",
        "output_data_type": "string",
        "purpose": "Getting local file path with upload file widget so that doc_load can use this path",
    },
    {
        "name": "ui_output_text",
        "description": "Shows text output to the user.",
        "good_at": "Showing text to the user.",
        "input_data_type": "*",
        "output_data_type": "none",
        "purpose": "Displaying textual information to the user.",
    },
    {
        "name": "prompt_template",
        "description": "Generate any string output according to the given instruction by AI",
        "good_at": "Creating context-aware string, responses, role play, instructions, that can be generated by AI",
        "input_data_type": "*string",
        "output_data_type": "string",
        "purpose": "Using AI to generate smart text output from given context or instruction",
    },
    {
        "name": "doc_loader",
        "description": "Load file content from path (notion zip file path or txt or url or pdf file path or csv file path or powerpoint file path or docx path or youtube url or excel file path) and generate docs",
        "good_at": "Loading from external sources only from url or path not the content",
        "input_data_type": "string",
        "output_data_type": "Document",
        "purpose": "Loading external files",
    },
    {
        "name": "doc_to_string",
        "description": "Convert Document object to string",
        "good_at": "Converting Document object to string where the next task is expecting string instead of Document object",
        "input_data_type": "Document",
        "output_data_type": "string",
        "purpose": "Converting Document object to string",
    },
    {
        "name": "string_to_doc",
        "description": "Convert string to Document object",
        "good_at": "Converting string to Document object where the next task is expecting Document object instead of string",
        "input_data_type": "string",
        "output_data_type": "Document",
        "purpose": "Converting string to Document object",
    },
    {
        "name": "ui_input_chat",
        "description": "Get user message/text input for conversation-based application",
        "good_at": "Getting text input from the user for chat-based application",
        "input_data_type": "none",
        "output_data_type": "string",
        "purpose": "For chat interface, get user text input. It does not need to be included multiple times",
    },
    {
        "name": "ui_output_chat",
        "description": "Display the conversation history in a chat-based application. It is the only thing that you can use for displaying the chat",
        "good_at": "Displaying chat history",
        "input_data_type": "string",
        "output_data_type": "none",
        "purpose": "For conversation-based apps, it displays the chat conversation with history.",
    },
    {
        "name": "chat",
        "description": "Chat version of prompt_template that can remember the conversation history while responding",
        "good_at": "Chatbot like applications and any application requiring chat property.",
        "input_data_type": "*string",
        "output_data_type": "string",
        "purpose": "For conversation-based apps, it generates the responses while remembering the conversation history",
    },
    {
        "name": "python",
        "description": """Implement and call generic python function from given description which can be done using the libraries: 
        [NumPy, Matplotlib, Seaborn, Scikit-Learn, NLTK, SciPy, OpenCV, Pandas]""",
        "good_at": "Writing generic python code.",
        "input_data_type": "*",
        "output_data_type": "*",
        "purpose": "It generates python code from general purpose instructions",
    },
    {
        "name": "plan_and_execute",
        "description": "It is intelligent AI agent that can answer any specific question on internet.",
        "good_at": "Applications requiring up to date knowledge on the internet.",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "By using internet, it autonomously give answer for any question available in the web. It can answer questions as specific as possible so you don't need to iterate over the answer.",
    },
    {
        "name": "search_chat",
        "description": "It is intelligent chat-based AI agent that can answer any specific question on internet.",
        "good_at": "Applications requiring up to date knowledge on the internet. It can also be used in chat app",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "By using internet, it autonomously give answer for any question available in the web. It can answer questions as specific as possible so you don't need to iterate over the answer. It also remembers the chat history while responsing",
    },
    {
        "name": "doc_summarizer",
        "description": "Summarize Document Objects",
        "good_at": "Summarizing long Document Objects",
        "input_data_type": "Document",
        "output_data_type": "string",
        "purpose": "Summarize long Document Objects",
    },
    {
        "name": "prompt_list_parser",
        "description": "Transform the input text into a list.",
        "good_at": "Transforming text into a list.",
        "input_data_type": "string",
        "output_data_type": "*",
        "purpose": "Converts textual data into structured list format.",
    },
    {
        "name": "router",
        "description": "When there are multiple prompt_template objects, it uses the appropriate one to answer the question.",
        "good_at": "Handling different types of questions that require different abilities.",
        "input_data_type": "*prompt_template",
        "output_data_type": "string",
        "purpose": "Routes queries to the appropriate handler based on context or type.",
    },
    {
        "name": "react",
        "description": "Answer questions that require external search on the web.",
        "good_at": "Answering questions that require Google search or other web searches.",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "Finds information online to answer user queries.",
    },
    {
        "name": "cpal_chain",
        "description": "Solve math problems end to end",
        "good_at": "Directly solving any math problems",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "Performing mathematical calculations and solving problems based on the input question",
    },
    {
        "name": "hub_bash",
        "description": "Do operations on the bash by running needed scripts on the terminal to apply the command.",
        "good_at": "Executing bash commands and providing results.",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "Running scripts or commands on the terminal and returning the output.",
    },
    {
        "name": "hub_meteo",
        "description": "Gives weather-related information from the question.",
        "good_at": "Answering weather-related questions.",
        "input_data_type": "string",
        "output_data_type": "string",
        "purpose": "Providing weather forecasts, conditions, and related information.",
    },
]


def jsonFixer(data):
    data = json.dumps(data, indent=4)
    return data.replace("{", "{{").replace("}", "}}")

def isTaskAvailable(task, app_chat, app_prompt_template, app_search, app_summary):
    if not app_chat:
        if "chat" in task["name"]:
            return False
    elif task["name"] == "python":
        return False
    elif task["name"] == "plan_and_execute":
        return False
    elif task["name"] == "prompt_template":
        return False
    elif app_search:
        if task["name"] == "chat":
            return False

    if not app_prompt_template:
        if task["name"] in [
            "prompt_template",
            "doc_loader",
            "doc_to_string",
            "string_to_doc"
        ]:
            return False
        
    if not app_summary:
        if task["name"] == "doc_summarizer":
            return False
        
    elif task["name"] == "python":
        return False

    if not app_search:
        if task["name"] == "plan_and_execute":
            return False
        if task["name"] == "search_chat":
            return False
        
    elif task["name"] == "python":
        return False

    return True


def getAvailableTasks(app_type):
    app_prompt_template = True  # neutral

    app_chat = app_type["is_chat"] == "true"
    app_search = app_type["is_search"] == "true"
    app_summary = app_type["is_summary"] == "true"
    app_prompt_template = app_type["is_ai"] == "true"

    tasks = []
    for task in ALL_TASKS[:AVAILABLE_TASKS_COUNT]:
        if isTaskAvailable(task, app_chat, app_prompt_template, app_search, app_summary):
            tasks.append(task)

    return tasks


def getTasks(app_type):
    TASKS = getAvailableTasks(app_type)
    TASK_TYPE2_TASK = {task["name"]: task for task in TASKS}

    TASK_NAMES = [task["name"] for task in TASKS]

    TASK_PURPOSES = {task["name"]: task["purpose"] for task in TASKS}
    TASK_PURPOSES = jsonFixer(TASK_PURPOSES)

    TASK_DESCRIPTIONS = jsonFixer(TASKS)

    TASK_DTYPES = {
        task["name"]: {
            "input_data_type": task["input_data_type"],
            "output_data_type": task["output_data_type"],
        }
        for task in TASKS
    }

    TASK_DTYPES = jsonFixer(TASK_DTYPES)

    return TASK_DESCRIPTIONS, TASK_NAMES, TASK_DTYPES, TASK_PURPOSES, TASK_TYPE2_TASK


def getPlanGenHelper(app_type):
    prompt_template_must = False
    app_chat_must = app_type["is_chat"] == "true"
    app_summarize_must = app_type["is_summary"] == "true"
    app_search_must = app_type["is_search"] == "true"
    if not (app_chat_must or app_summarize_must or app_search_must):
        if app_type["is_ai"] == "true":
            prompt_template_must = True

    helper = ""
    if prompt_template_must:
        helper += "Since the application is AI-based, you must use 'prompt_template' task in the steps.\n"
    if app_summarize_must:
        helper += "Since the application requires summarization, you must use 'doc_summarizer' task in the steps.\n"
    if app_search_must:
        if app_chat_must:
            helper += "Since the application requires up to date knowledge in the web, you must use 'search_chat' task in the steps.\n"
        else:
            helper += "Since the application requires up to date knowledge in the web, you must use 'plan_and_execute' task in the steps.\n"
            
    if app_chat_must:
        if app_search_must:
            helper += "Since the application is chat-based, you must use 'ui_input_chat' and 'ui_output_chat' and 'search_chat' tasks in the steps.\n"
        else:
            helper += "Since the application is chat-based, you must use 'ui_input_chat' and 'ui_output_chat' and 'chat' tasks in the steps.\n"

    return helper



system_template = """
You are an AI agent that can generate single like placeholder text depending on the instruction and the variable name.
"""

human_template = """
#############################
variable: url
instruction: Get website URL from the user
placeholder:Enter website URL
#############################
variable: source_text
instruction: Get source text from the user
placeholder:Type the source text
#############################
variable: input_language
instruction: Get the input language from the user
placeholder:Enter the input language
#############################
variable: {variable}
instruction: {instruction}
placeholder:
"""

code = """
for message in st.session_state.messages:
    with st.chat_message(message["role"]):  
        st.markdown(message["content"])
        
if {variable} := st.chat_input("{placeholder}"):
    with st.chat_message("user"):
        st.markdown({variable})
    st.session_state.messages.append({{"role": "user", "content": {variable}}})
"""



system_template = """
You are helpful assistant for checking if the "Generated Tasks" are correct in terms of input_data_type, output_data_type and their connections.
These are the important keys of task objects that you will analyze:

task_type: Should match one of the task names provided in task descriptions.
task_name: Define a specific name for the task that aligns with the corresponding plan step.
input_key: List the "output_key" values from parent tasks used as input or "none" if there's no input or if it comes from the user.
input_data_type: The list of data types of the inputs
output_key: Designate a unique key for the task's output. It is compatible with the output type if not none
output_data_type: The data type of the output

You will check if all the generated tasks' input_data_type and output_data_type are compatible with the original tasks.

These are the original tasks that you will compare with:

"Original Tasks": {TASK_DESCRIPTIONS}

You will create a JSON object with the following 2 keys:

feedback: List of feedbacks for each task by comparing it with the task in the "Original Tasks"
valid: It is a boolean value that indicates whether these tasks are valid (no problem) or not.
"""

human_template = """
"Generated Tasks" : {tasks}
JSON:
"""



system_template = """
You will summarization code with a strict structure like in the below but 
loader will change depending on the input
###
from langchain.chat_models import ChatOpenAI
from langchain.chains.summarize import load_summarize_chain
def {function_name}(docs):
    llm = ChatOpenAI(temperature=0, model_name="gpt-3.5-turbo-16k", openai_api_key=openai_api_key)
    chain = load_summarize_chain(llm, chain_type="stuff")
    return chain.run(docs)
if {argument}:
    {variable} = summarize(argument)
else:
    variable = ""
###
"""

human_template = """
Here is the part of the code that you are supposed to continue:
{code_snippets}

Write a summarize function for the argument name and variable below:
Argument Name : {argument}
Variable Name : {variable}
Summarization Code:
"""

imports = """
from langchain.chat_models import ChatOpenAI
from langchain.chains.summarize import load_summarize_chain
"""

functions = """
def {function_name}({argument}):
    llm = ChatOpenAI(temperature=0, model_name="gpt-3.5-turbo-16k", openai_api_key=openai_api_key)
    chain = load_summarize_chain(llm, chain_type="stuff")
    with st.spinner('DemoGPT is working on it. It might take 5-10 seconds...'):
        return chain.run({argument})
"""


imports = """
from langchain.docstore.document import Document
"""
outputs = """
{variable} =  [Document(page_content={argument}, metadata={{'source': 'local'}})]
"""

outputs = """
if not openai_api_key.startswith('sk-'):
    st.warning('Please enter your OpenAI API key!', icon='⚠')
    {variable} = ""
elif {argument}:
    {variable} = {function_name}({argument})
else:
    variable = ""
"""


system_template = """
Generate a feedback JSON to the given plan which is prepared for the given instruction.
The JSON includes 2 keys which are "success" and "feedback".
"feedback" corresponds to the feedback to the plan.
"success" corresponds to the success of the plan. If the plan is good then "success" should be True. Otherwise, it should be False. 

In each step, there are tasks in the below format:
[$task_name($args) ---> $output]

You should check 2 things.

1.Only those task names are allowed to be used:
{TASK_NAMES}

2. If one of the task's output is  input of another, then output_data_type of previous one
should be the same as input_data_type of successor.
These are the data types:
{TASK_DTYPES}
"""

human_template = """
Instruction: {instruction}
Plan:{plan}
Feedback JSON:
"""



system_template = """
You are a good Python developer and your task is creating and calling a function called {function_name},
which converts Document object to a string.
You can easily convert Document object docs to string like in the following:

{variable} = "\".join([doc.page_content for doc in {argument}])

Use that function according to the given argument and variable name.
You will assign {variable} to the string version of {argument}.

Here is the part of the code that you are supposed to continue:
{code_snippets}
"""

human_template = """
Argument:{argument}
Variable:{variable}
Python Code:
"""

outputs = '{variable} = "".join([doc.page_content for doc in {argument}])'



from . import (about, app_type, combine, combine_v2, feedback, final,
               how_to_use, imports, plan, plan_feedback, plan_refiner,
               plan_with_inputs, prompt_chat_refiner, refine, system_inputs,
               task_controller, task_refiner, tasks, title)
from .self_refinement import final_refiner
from .task_list import (chat, doc_load, doc_to_string, string_to_doc, path_to_file,
                        prompt_template, python_coder, search, search_chat, summarize,
                        ui_input_chat, ui_output_chat, ui_input_file, ui_input_text,
                        ui_output_text, detailed_description)



import os
import textwrap
from time import sleep

import autopep8
from openai import OpenAI
import streamlit as st
from tqdm import trange

from demogpt.chains.chains import Chains
from demogpt.chains.task_chains import TaskChains
from demogpt.chains.task_chains_seperate import TaskChainsSeperate
from demogpt.utils import (getCodeSnippet, getCodeSnippetSeperate,
                           getFunctionNames, init, initSeperate,
                           reorderTasksForChatApp)


class DemoGPT:
    def __init__(
        self,
        openai_api_key=os.getenv("OPENAI_API_KEY", ""),
        model_name="gpt-3.5-turbo-0613",
        max_steps=10,
        plan_max_steps = 3,
        openai_api_base="",
    ):
        assert len(
            openai_api_key.strip()
        ), "Either give openai_api_key as an argument or put it in the environment variable"
        self.model_name = model_name
        self.openai_api_key = openai_api_key
        self.max_steps = max_steps  # max iteration for refining the model purpose
        self.plan_max_steps = plan_max_steps # max iteration for refining the plan
        self.openai_api_base = openai_api_base
        self.FAIL_MESSAGE = """🚀✨ Impressive! While DemoGPT can handle a galaxy of app ideas, 
        you've shot for the stars with a unique one. We're ramping up our engines to meet such visionary requests. 
        Give us a little time, and we'll be right there with you.\n\n📧
        Care to leave your email? We'll notify you when we're ready for this stellar journey!"""
        
        self.gpt4_message = """
🚫 Access Denied to {model_name}!

Hey there! It looks like you're trying to create an app with {model_name}, but unfortunately, you don't have the required access. Fear not! You can:

Upgrade your access through OpenAI's platform.
Opt for another model that you have access to and give it another whirl.
We appreciate your understanding and look forward to seeing what you create! 😊
            """
        self.available_models = self.get_available_models(openai_api_key)
    
    @classmethod
    def get_available_models(cls, openai_api_key):
        if not openai_api_key:
            return []
        try:
            client = OpenAI(api_key = openai_api_key)
            models = client.models.list()
            return [model.id for model in models.data if "gpt" in model.id]
        except Exception as e:
            return []
        
        
    @property
    def hasGPT4(self):
        return "gpt-4-0613" in self.available_models
                
    def setModel(self, model_name):
        self.model_name = model_name
        
        assert self.model_name, f"No model is selected, the selected model is {self.model_name}" 
        assert self.model_name in self.available_models , self.gpt4_message.format(model_name=self.model_name)
                
        Chains.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base, has_gpt4=self.hasGPT4
        )
        TaskChains.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base
        )
        TaskChainsSeperate.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base
        )
        Chains.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base, has_gpt4=self.hasGPT4
        )
        TaskChains.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base
        )
        TaskChainsSeperate.setLlm(
            self.model_name, self.openai_api_key, openai_api_base=self.openai_api_base
        )

    def __repr__(self) -> str:
        return f"DemoGPT(model_name='{self.model_name}',max_steps={self.max_steps})"

    def callForChat(
        self,
        instruction="Create a translation system that converts English to French",
        title="",
        app_type={}
    ):
        
        yield {
            "stage": "system_inputs",
            "completed": False,
            "percentage": 0,
            "done": False,
            "message": "System inputs are being detected...",
            "failed": False,
        }

        system_inputs = Chains.systemInputs(instruction=instruction)

        yield {
            "stage": "plan",
            "completed": False,
            "percentage": 10,
            "done": False,
            "message": "Plan creation has started...",
            "failed": False,
        }

        plan = Chains.planWithInputs(
            instruction=instruction, system_inputs=system_inputs, app_type=app_type
        )

        yield {
            "stage": "plan",
            "completed": True,
            "percentage": 20,
            "done": False,
            "message": "Plan has been generated.",
            "failed": False,
        }

        sleep(1)

        yield {
            "stage": "plan_controlling",
            "completed": True,
            "percentage": 25,
            "done": False,
            "message": "Plan is being controlled.",
            "failed": False,
        }

        plan_controller_result = Chains.planController(plan=plan, app_type=app_type)

        for _ in trange(self.plan_max_steps):
            if not plan_controller_result["valid"]:
                plan = Chains.planRefiner(
                    instruction=instruction,
                    plan=plan,
                    feedback=plan_controller_result["feedback"],
                    app_type=app_type,
                )
                plan_controller_result = Chains.planController(
                    plan=plan, app_type=app_type
                )
            else:
                break

        yield {
            "stage": "task",
            "completed": False,
            "percentage": 30,
            "done": False,
            "message": "Task generation has started...",
            "failed": False,
        }

        task_list = Chains.tasks(
            instruction=instruction, plan=plan, app_type=app_type
        )

        yield {
            "stage": "task",
            "completed": True,
            "percentage": 50,
            "done": False,
            "message": "Tasks have been generated.",
            "tasks": task_list,
            "failed": False,
        }

        sleep(1)

        yield {
            "stage": "task_controlling",
            "completed": True,
            "percentage": 55,
            "done": False,
            "message": "Tasks are being controlled.",
            "failed": False,
        }

        task_controller_result = Chains.taskController(
            tasks=task_list, app_type=app_type
        )

        for _ in trange(self.max_steps):
            if not task_controller_result["valid"]:
                try:
                    task_list = Chains.refineTasks(
                        instruction=instruction,
                        tasks=task_list,
                        feedback=task_controller_result["feedback"],
                        app_type=app_type,
                    )
                    task_controller_result = Chains.taskController(
                        tasks=task_list, app_type=app_type
                    )
                except Exception as e:
                    print(e)
                    if "16k" in Chains.model:
                        break
                    st.toast(
                        "To increase the window size, changing model type to gpt-3.5-turbo-16k-0613"
                    )
                    Chains.setModel("gpt-3.5-turbo-16k-0613")
            else:
                break

        if not task_controller_result["valid"]:

            yield {
                "stage": "task",
                "completed": False,
                "percentage": 100,
                "done": False,
                "message": self.FAIL_MESSAGE,
                "failed": True,
            }

        else:
            
            title = Chains.title(instruction=instruction)
            
            task_list = reorderTasksForChatApp(task_list) # for chat apps, remove the code between chat input and chat output

            code_snippets = init(title)

            sleep(1)

            yield {
                "stage": "draft",
                "completed": False,
                "percentage": 60,
                "done": False,
                "message": "Converting tasks to code snippets...",
                "title":title
            }

            num_of_tasks = len(task_list)

            for i, task in enumerate(task_list):
                code = getCodeSnippet(instruction, task, code_snippets, self.max_steps)
                code = "#" + task["description"] + "\n" + code
                code_snippets += code
                yield {
                    "stage": "draft",
                    "completed": i + 1 == num_of_tasks,
                    "percentage": 60 + int(20 * (i + 1) / num_of_tasks),
                    "done": False,
                    "message": f"{i+1}/{num_of_tasks} tasks have been converted to code",
                    "code": code,
                    "title":title
                }

            sleep(1)

            yield {
                "stage": "draft",
                "completed": False,
                "percentage": 85,
                "done": False,
                "message": "Code snippets are being combined...",
                "title":title
            }

            final_code = code_snippets
            # finalize the format
            final_code = autopep8.fix_code(final_code)
            
            final_code = Chains.addAboutAndHTU(instruction, title, final_code, plan)

            yield {
                "stage": "final",
                "completed": True,
                "percentage": 100,
                "done": True,
                "message": "Final code has been generated. Directing to the demo page...",
                "code": final_code,
                "title":title
            }
            
    def __call__(self, instruction, title=""):
        try:
            for data in self.run(instruction):
                yield data  
        except Exception as e:
            print(e)
            yield {
                "stage": "task",
                "completed": False,
                "percentage": 100,
                "done": False,
                "message": self.FAIL_MESSAGE,
                "failed": True
                }
    def run(
        self,
        instruction="Create a translation system that converts English to French",
        title="",
    ):
        
        def getCode(imports, functions, prefix, inputs, outputs, how_to, about, title):
            return f"""
{imports}

{functions}

{prefix}

{how_to}

{about}

with st.form(key="form"):
    st.title('{title}')
{inputs}
    submit_button = st.form_submit_button(label='Submit')
    if not openai_api_key.startswith('sk-'):
        st.warning('Please enter your OpenAI API key!', icon='⚠')
    if submit_button:
{outputs}
"""
        yield {
            "stage": "system_inputs",
            "completed": False,
            "percentage": 0,
            "done": False,
            "message": "System inputs are being detected...",
            "failed": False,
        }

        app_type = Chains.appType(instruction=instruction)
        
        if app_type["is_chat"] == "true":
            for data in self.callForChat(instruction, app_type=app_type):
                yield data            
        else:
        
            system_inputs = Chains.systemInputs(instruction=instruction)

            yield {
                "stage": "plan",
                "completed": False,
                "percentage": 10,
                "done": False,
                "message": "Plan creation has started...",
                "failed": False,
            }
            
            sleep(10)

            plan = Chains.planWithInputs(
                instruction=instruction, system_inputs=system_inputs, app_type=app_type
            )

            yield {
                "stage": "plan",
                "completed": True,
                "percentage": 20,
                "done": False,
                "message": "Plan has been generated.",
                "failed": False,
            }

            sleep(1)

            yield {
                "stage": "plan_controlling",
                "completed": True,
                "percentage": 25,
                "done": False,
                "message": "Plan is being controlled.",
                "failed": False,
            }

            plan_controller_result = Chains.planController(plan=plan, app_type=app_type)

            for _ in trange(self.plan_max_steps):
                sleep(10)
                if not plan_controller_result["valid"]:
                    plan = Chains.planRefiner(
                        instruction=instruction,
                        plan=plan,
                        feedback=plan_controller_result["feedback"],
                        app_type=app_type,
                    )
                    plan_controller_result = Chains.planController(
                        plan=plan, app_type=app_type
                    )
                else:
                    break

            yield {
                "stage": "task",
                "completed": False,
                "percentage": 30,
                "done": False,
                "message": "Task generation has started...",
                "failed": False,
            }

            task_list = Chains.tasks(
                instruction=instruction, plan=plan, app_type=app_type
            )

            yield {
                "stage": "task",
                "completed": True,
                "percentage": 50,
                "done": False,
                "message": "Tasks have been generated.",
                "tasks": task_list,
                "failed": False,
            }

            sleep(1)

            yield {
                "stage": "task_controlling",
                "completed": True,
                "percentage": 55,
                "done": False,
                "message": "Tasks are being controlled.",
                "failed": False,
            }

            task_controller_result = Chains.taskController(
                tasks=task_list, app_type=app_type
            )

            for _ in trange(self.max_steps):
                sleep(10)
                if not task_controller_result["valid"]:
                    try:
                        task_list = Chains.refineTasks(
                            instruction=instruction,
                            tasks=task_list,
                            feedback=task_controller_result["feedback"],
                            app_type=app_type,
                        )
                        task_controller_result = Chains.taskController(
                            tasks=task_list, app_type=app_type
                        )
                    except Exception as e:
                        print(e)
                        if "16k" in Chains.model:
                            break
                        st.toast(
                            "To increase the window size, changing model type to gpt-3.5-turbo-16k-0613"
                        )
                        Chains.setModel("gpt-3.5-turbo-16k-0613")
                else:
                    break

            if not task_controller_result["valid"]:

                yield {
                    "stage": "task",
                    "completed": False,
                    "percentage": 100,
                    "done": False,
                    "message": self.FAIL_MESSAGE,
                    "failed": True,
                }

            else:
                
                imports = ""
                functions = ""
                inputs = ""
                outputs = ""
                
                title = Chains.title(instruction=instruction)
                
                task_list = reorderTasksForChatApp(task_list) # for chat apps, remove the code between chat input and chat output
                
                res = initSeperate(title)
                
                prefix = res["prefix"]
                
                imports = res["imports"]
                
                code_snippets = res["code"]
                
                sleep(1)

                yield {
                    "stage": "draft",
                    "completed": False,
                    "percentage": 60,
                    "done": False,
                    "message": "Converting tasks to code snippets...",
                    "title":title
                }

                num_of_tasks = len(task_list)

                for i, task in enumerate(task_list):
                    res = getCodeSnippetSeperate(instruction, task, code_snippets, self.max_steps)
                    code = "#" + task["description"] + "\n" + res["code"]
                    
                    if res["imports"] not in imports:
                        imports += res["imports"] + "\n"
                    functions += res["functions"] + "\n"
                    inputs += res["inputs"] + "\n"
                    outputs += res["outputs"] + "\n"
                    
                    code_snippets += code
                    
                    yield {
                        "stage": "draft",
                        "completed": i + 1 == num_of_tasks,
                        "percentage": 60 + int(20 * (i + 1) / num_of_tasks),
                        "done": False,
                        "message": f"{i+1}/{num_of_tasks} tasks have been converted to code",
                        "code": code,
                        "title":title
                    }

                sleep(1)

                yield {
                    "stage": "draft",
                    "completed": False,
                    "percentage": 85,
                    "done": False,
                    "message": "Code snippets are being combined...",
                    "title":title
                }
                
                inputs = textwrap.indent(inputs, 4*' ')
                outputs = textwrap.indent(outputs, 8*' ')
                
                how_to, about = Chains.getAboutAndHTU(instruction, title, plan)
                
                final_code = getCode(imports, functions, prefix, inputs, outputs, how_to, about, title)
                # finalize the format
                final_code = autopep8.fix_code(final_code)

                yield {
                    "stage": "final",
                    "completed": True,
                    "percentage": 100,
                    "done": True,
                    "message": "Final code has been generated. Directing to the demo page...",
                    "code": final_code,
                    "title":title
                }


system_template = """
You are an AI assistant that can generate JSON with 2 keys which are title and data_type
The values correspond to those keys will be used in the streamlit's file_uploader function like in the below:
st.file_uploader($title,type=$data_type)
So title should be string and data_type should be array of data types such as "txt", "csv" ...
You will decide title and data_type by only considering the instruction given to you.
You need to know that if instruction include Notion file, it means the data type should be zip because Notion export is directly a zip file.
"""

human_template = """
Instruction:{instruction}
JSON:
"""

code = """
uploaded_file = st.file_uploader("{title}", type={data_type}, key='{variable}')
if uploaded_file is not None:
    # Create a temporary file to store the uploaded content
    extension = uploaded_file.name.split(".")[-1]
    with tempfile.NamedTemporaryFile(delete=False, suffix=f'.{{extension}}') as temp_file:
        temp_file.write(uploaded_file.read())
        {variable} = temp_file.name # it shows the file path
else:
    {variable} = ''
        """
        
inputs = """
uploaded_file = st.file_uploader("{title}", type={data_type}, key='{variable}')
"""

outputs = """
if uploaded_file is not None:
    # Create a temporary file to store the uploaded content
    extension = uploaded_file.name.split(".")[-1]
    with tempfile.NamedTemporaryFile(delete=False, suffix=f'.{{extension}}') as temp_file:
        temp_file.write(uploaded_file.read())
        {variable} = temp_file.name # it shows the file path
else:
    {variable} = ''
"""



system_template = """
Create a plan to fulfill the given instruction. 
The plan should be broken down into clear, logical steps that detail how to accomplish the task. 
Consider all necessary user interactions, system processes, and validations, 
and ensure that the steps are in a logical sequence that corresponds to the given instruction.
Don't generate impossible steps in the plan because only those tasks are available:
{TASK_DESCRIPTIONS}

Pay attention to the input_data_type and the output_data_type.
If one of the task's output is  input of another, then output_data_type of previous one
should be the same as input_data_type of successor.

Only those task types are allowed to be used:
{TASK_NAMES}

Highly pay attention to the input data type and the output data type of the tasks while creating the plan. These are the data types:

{TASK_DTYPES}

When you create a step in the plan, its input data type 
either should be none or the output data type of the caller step. 

If you use a task in a step, highly pay attention to the input data type and the output data type of the task because it should be compatible with the step.

"""

human_template = """
Don't generate redundant steps which is not meant in the instruction.


Instruction: Application that can analyze the user
System Inputs: []
Let's work this out in a step by step way to be sure we have the right answer.
1. Generate question to understand the personality of the user by 'prompt_template'
2. Show the question to the user by 'ui_output_text'
3. Get answer from the user for the asked question by 'ui_input_text'
4. Analyze user's answer by 'prompt_template'.
5. Show the result to the user by 'ui_input_text'.

Instruction: Create a system that can summarize a powerpoint file
System Inputs:[powerpoint_file]
Let's work this out in a step by step way to be sure we have the right answer.
1. Get file path from the user by 'ui_input_file' for the powerpoint file
2. Use 'doc_loader' to load the powerpoint file as Document from the file path.
3. Use 'doc_summarizer' to generate summarization from the Document. 
5. If summarization is ready, display it to the user by 'ui_output_text'.

Instruction: Create a translator which translates to any language
System Inputs:[output_language, source_text]
Let's work this out in a step by step way to be sure we have the right answer.
1. Get output language from the user by 'ui_input_text'
2. Get source text which will be translated from the user by 'ui_input_text'
3. If all the inputs are filled, use 'prompt_template' to translate text to output language
4. If translated text is ready, show it to the user by 'ui_output_text'

Instruction: Generate a system that can generate tweet from hashtags and give a score for the tweet.
System Inputs:[hashtags]
Let's work this out in a step by step way to be sure we have the right answer.
1. Get hashtags from the user by 'ui_input_text'
2. If hashtags are filled, use 'prompt_template' to create tweet.
3. If tweet is created, use 'prompt_template' to generate a score from the tweet.
4. If score is created, display tweet and score to the user by 'ui_output_text'.

Instruction: Summarize a text taken from the user
System Inputs:[text]
Let's work this out in a step by step way to be sure we have the right answer.
1. Get text from the user by 'ui_input_text' 
2. Use 'prompt_template' to summarize the given text.
3. If summarization is ready, display it to the user by 'ui_output_text'.

Instruction: Create a platform which lets the user select a lecture and then show topics for that lecture 
then give a question to the user. After user gives his/her answer, it gives a score for the answer and give explanation.
System Inputs:[lecture, topic, user_answer]
Let's work this out in a step by step way to be sure we have the right answer.
1. Use 'prompt_template' to generate lectures
2. Among those generated by prompt_template, get lecture from the user by 'ui_input_text'.
3. After user selects a lecture, generate topics releated to that lecture by 'prompt_template'.
4. Among those generated by prompt_template, get topic from the user by 'ui_input_text' .
5. After user selects the topic, use 'prompt_template' to generate a question related to that topic and lecture
6. Get answer from the user by 'ui_input_text'.
7. Use 'prompt_template' to generate the real answer and score for the user's answer.
8. Display real and answer and score for the user's answer by 'ui_output_text'.

Instruction: Create a system that can generate blog post related to a website
System Inputs: [url]
Let's work this out in a step by step way to be sure we have the right answer.
1. Get website URL from the user by 'ui_input_text'
2. Use 'doc_loader' to load the website as Document from URL
3. Use 'doc_to_string' to convert Document to string content
4. If string content is generated, use 'prompt_template' to generate a blog post related to that string content.
5. If blog post is generated, display it to the user by 'ui_output_text'.

Instruction: {instruction}
Let's work this out in a step by step way to be sure we have the right answer.
"""
